@@ -4,10 +4,13 @@ use strict;
 use warnings;
 use Exporter 'import';
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 our @EXPORT_OK = qw(
   cols_non_empty
   non_unique_cols
+  key_to_index
+  make_pk_map
+  pk_col_counts
   chop_lines
   chop_cols
   header_merge
@@ -79,6 +82,46 @@ sub non_unique_cols {
   \%col2count;
 }
 
+sub key_to_index {
+  my ($row) = @_;
+  +{ map +($row->[$_] => $_), 0..$#$row };
+}
+
+sub make_pk_map {
+  my ($data, $pk_colkey, $other_colkeys) = @_;
+  my $k2i = key_to_index($data->[0]);
+  my @invalid = grep !defined $k2i->{$_}, $pk_colkey, @$other_colkeys;
+  die "Invalid keys (@invalid)" if @invalid;
+  my $pk_colnum = $k2i->{$pk_colkey};
+  my %altcol2value2pk;
+  for my $i (1..$#$data) {
+    my $row = $data->[$i];
+    next if !length(my $pk_val = $row->[$pk_colnum]);
+    for my $alt_k ($pk_colkey, @$other_colkeys) {
+      next if !length(my $alt_v = $row->[$k2i->{$alt_k}]);
+      $altcol2value2pk{$alt_k}{$alt_v} = $pk_val;
+    }
+  }
+  \%altcol2value2pk;
+}
+
+sub pk_col_counts {
+  my ($data, $pk_map) = @_;
+  my $k2i = key_to_index($data->[0]);
+  my (%col2code2exact, @no_exact_match);
+  for my $i (1..$#$data) {
+    my ($row, $exact_match) = $data->[$i];
+    for my $possible_col (keys %$k2i) {
+      my $val = $row->[ $k2i->{$possible_col} ];
+      my @match_codes_yes = grep exists $pk_map->{$_}{$val}, keys %$pk_map;
+      $col2code2exact{$possible_col}{$_}++ for @match_codes_yes;
+      $exact_match ||= @match_codes_yes;
+    }
+    push @no_exact_match, $row if !$exact_match;
+  }
+  (\%col2code2exact, \@no_exact_match);
+}
+
 1;
 
 =encoding utf8
@@ -232,6 +275,31 @@ columns with only a couple of entries, which are more usefully chopped.
 Takes the first row of the given data, and returns a hash-ref mapping
 any non-unique column-names to the number of times they appear.
 
+=head2 key_to_index
+
+Given an array-ref (probably the first row of a CSV file, i.e. column
+headings), returns a hash-ref mapping the cell values to their zero-based
+index.
+
+=head2 make_pk_map
+
+  my $altcol2value2pk = make_pk_map($data, $pk_colkey, \@other_colkeys);
+
+Given C<$data>, the heading of the primary-key column, and an array-ref
+of headings of alternative key columns, returns a hash-ref mapping each
+of those alternative key columns (plus the C<$pk_colkey>) to a map from
+that column's value to the relevant row's primary-key value.
+
+=head2 pk_col_counts
+
+  my ($colname2potential_key2count, $no_exact_match) = pk_col_counts($data, $pk_map);
+
+Given C<$data> and a primary-key (etc) map created by the above, returns
+a tuple of a hash-ref mapping each column that gave any matches to a
+further hash-ref mapping each of the potential key columns given above
+to how many matches it gave, and an array-ref of rows that had no exact
+matches.
+
 =head1 SEE ALSO
 
 L<Text::CSV>
